#include <windows.h>
#include "config.h"
#include "ui.h"

static int str2int(const char* buf) {
	int result=0;
	int minus=0;
	if(*buf=='-') {
		minus=1;
		buf++;
	}
	while(*buf!=0 && *buf>='0' && *buf<='9') {
		result=result*10+(*buf-'0');
		buf++;
	}
	if(minus)return -result; else return result;
}

static char exedir[1024]={0};

const char* getExeDir(void) {
	if(exedir[0]==0) {
		int i,j,len;
		unsigned char* exedir2=(unsigned char*)exedir;
		GetModuleFileName(NULL,(CHAR*)exedir,sizeof(exedir));
		len=lstrlen(exedir);
		j=len-1;
		for(i=0;i<len;i++) {
			if((exedir2[i]>=0x81 && exedir2[i]<=0x9F) ||
				(exedir2[i]>=0xE0 && exedir2[i]<=0xEF))i++;
			else if(exedir[i]=='\\')j=i;
		}
		exedir[j+1]=0;
	}
	return exedir;
}

void loadConfig(config_t* config) {
	char inifile[1024];
	lstrcpy(inifile,getExeDir());
	lstrcat(inifile,"sokudokensei.ini");
	config->useRegistry=GetPrivateProfileInt("registry","useregistry",0,inifile);
	if(config->useRegistry) {
		HKEY hKey;
		if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\mikeCAT\\sokudokensei",0,
				KEY_QUERY_VALUE,&hKey)==ERROR_SUCCESS) {
			DWORD size;
			size=sizeof(config->readMax);
			if(RegQueryValueEx(hKey,"readMax",0,NULL,(LPBYTE)&config->readMax,&size)!=ERROR_SUCCESS) {
				config->readMax=16;
			}
			size=sizeof(config->noReadOverMax);
			if(RegQueryValueEx(hKey,"noReadOverMax",0,NULL,(LPBYTE)&config->noReadOverMax,&size)!=ERROR_SUCCESS) {
				config->noReadOverMax=0;
			}
			size=sizeof(config->ignoreComma);
			if(RegQueryValueEx(hKey,"ignoreComma",0,NULL,(LPBYTE)&config->ignoreComma,&size)!=ERROR_SUCCESS) {
				config->ignoreComma=1;
			}
			size=sizeof(config->stopOnPeriod);
			if(RegQueryValueEx(hKey,"stopOnPeriod",0,NULL,(LPBYTE)&config->stopOnPeriod,&size)!=ERROR_SUCCESS) {
				config->stopOnPeriod=1;
			}
			size=sizeof(config->enableVoice);
			if(RegQueryValueEx(hKey,"enableVoice",0,NULL,(LPBYTE)&config->enableVoice,&size)!=ERROR_SUCCESS) {
				config->enableVoice=0;
			}
			size=sizeof(config->voiceFile);
			if(RegQueryValueEx(hKey,"voiceFile",0,NULL,(LPBYTE)&config->voiceFile,&size)!=ERROR_SUCCESS) {
				lstrcpy(config->voiceFile,"prime.wav");
			}
			RegCloseKey(hKey);
		} else {
			config->readMax=16;
			config->noReadOverMax=0;
			config->ignoreComma=1;
			config->stopOnPeriod=1;
			config->enableVoice=0;
			lstrcpy(config->voiceFile,"prime.wav");
		}
	} else {
		config->readMax=GetPrivateProfileInt("config","readMax",16,inifile);
		config->noReadOverMax=GetPrivateProfileInt("config","noReadOverMax",0,inifile);
		config->ignoreComma=GetPrivateProfileInt("config","ignoreComma",1,inifile);
		config->stopOnPeriod=GetPrivateProfileInt("config","stopOnPeriod",1,inifile);
		config->enableVoice=GetPrivateProfileInt("config","enableVoice",0,inifile);
		GetPrivateProfileString("config","voiceFile","prime.wav",config->voiceFile,
			sizeof(config->voiceFile),inifile);
	}
}

void saveConfig(const config_t* config) {
	if(config->useRegistry) {
		HKEY hKey;
		if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\mikeCAT\\sokudokensei",0,NULL,
				REG_OPTION_NON_VOLATILE,KEY_SET_VALUE,NULL,&hKey,NULL)==ERROR_SUCCESS) {
			RegSetValueEx(hKey,"readMax",0,REG_DWORD,(LPBYTE)&config->readMax,sizeof(config->readMax));
			RegSetValueEx(hKey,"noReadOverMax",0,REG_DWORD,(LPBYTE)&config->noReadOverMax,sizeof(config->noReadOverMax));
			RegSetValueEx(hKey,"ignoreComma",0,REG_DWORD,(LPBYTE)&config->ignoreComma,sizeof(config->ignoreComma));
			RegSetValueEx(hKey,"stopOnPeriod",0,REG_DWORD,(LPBYTE)&config->stopOnPeriod,sizeof(config->stopOnPeriod));
			RegSetValueEx(hKey,"enableVoice",0,REG_DWORD,(LPBYTE)&config->enableVoice,sizeof(config->enableVoice));
			RegSetValueEx(hKey,"voiceFile",0,REG_SZ,(LPBYTE)&config->voiceFile,lstrlen(config->voiceFile)+1);
			RegCloseKey(hKey);
		}
	} else {
		char inifile[1024];
		char strBuffer[1024];
		lstrcpy(inifile,getExeDir());
		lstrcat(inifile,"sokudokensei.ini");
		wsprintf(strBuffer,"%d",config->readMax);
		WritePrivateProfileString("config","readMax",strBuffer,inifile);
		wsprintf(strBuffer,"%d",config->noReadOverMax);
		WritePrivateProfileString("config","noReadOverMax",strBuffer,inifile);
		wsprintf(strBuffer,"%d",config->ignoreComma);
		WritePrivateProfileString("config","ignoreComma",strBuffer,inifile);
		wsprintf(strBuffer,"%d",config->stopOnPeriod);
		WritePrivateProfileString("config","stopOnPeriod",strBuffer,inifile);
		wsprintf(strBuffer,"%d",config->enableVoice);
		WritePrivateProfileString("config","enableVoice",strBuffer,inifile);
		WritePrivateProfileString("config","voiceFile",config->voiceFile,inifile);
	}
}

void applyConfigToUI(HWND hWnd,const config_t* config) {
	char strBuffer[1024];
	wsprintf(strBuffer,"%d",config->readMax);
	SetWindowText(GetDlgItem(hWnd,IDC_EDIT_READMAX),strBuffer);
	SendMessage(GetDlgItem(hWnd,IDC_COMBO_NOREADOVERMAX),CB_SETCURSEL,!!config->noReadOverMax,0);
	SendMessage(GetDlgItem(hWnd,IDC_CHECK_IGNORECOMMA),BM_SETCHECK,config->ignoreComma,0);
	SendMessage(GetDlgItem(hWnd,IDC_CHECK_STOPONPERIOD),BM_SETCHECK,config->stopOnPeriod,0);
	SendMessage(GetDlgItem(hWnd,IDC_CHECK_ENABLEVOICE),BM_SETCHECK,config->enableVoice,0);
	SetWindowText(GetDlgItem(hWnd,IDC_EDIT_VOICEFILE),config->voiceFile);
}

void getConfigFromUI(HWND hWnd,config_t* config) {
	char strBuffer[1024];
	GetWindowText(GetDlgItem(hWnd,IDC_EDIT_READMAX),strBuffer,sizeof(strBuffer));
	config->readMax=str2int(strBuffer);
	config->noReadOverMax=SendMessage(GetDlgItem(hWnd,IDC_COMBO_NOREADOVERMAX),CB_GETCURSEL,0,0);
	config->ignoreComma=SendMessage(GetDlgItem(hWnd,IDC_CHECK_IGNORECOMMA),BM_GETCHECK,0,0);
	config->stopOnPeriod=SendMessage(GetDlgItem(hWnd,IDC_CHECK_STOPONPERIOD),BM_GETCHECK,0,0);
	config->enableVoice=SendMessage(GetDlgItem(hWnd,IDC_CHECK_ENABLEVOICE),BM_GETCHECK,0,0);
	GetWindowText(GetDlgItem(hWnd,IDC_EDIT_VOICEFILE),config->voiceFile,sizeof(config->voiceFile));
}
