#define _WIN32_IE 0x0500
#include <windows.h>
#include "cpb_main.h"
#include "config.h"
#include "ui.h"
#include "makeui.h"

const LPCSTR szWindowClass = "SOKUDOKENSEI_WINDOW_CLASS";
const LPCSTR szWindowTitle = "y Ver1.00";

#define TIMER_UPDATE_CBC 100

#define MSG_TASKTRAY (WM_APP+1)
#define TASKTRAY_ICON_UID 200

HICON hIconSm;

void makeTaskTrayIcon(HWND hWnd) {
	NOTIFYICONDATA nfi={0};
	nfi.cbSize=sizeof(nfi);
	nfi.hWnd=hWnd;
	nfi.uID=TASKTRAY_ICON_UID;
	nfi.uFlags=NIF_MESSAGE | NIF_ICON | NIF_TIP;
	nfi.uCallbackMessage=MSG_TASKTRAY;
	nfi.hIcon=hIconSm;
	lstrcpy(nfi.szTip,"y");
	Shell_NotifyIcon(NIM_ADD,&nfi);
}

void deleteTaskTrayIcon(HWND hWnd) {
	NOTIFYICONDATA nfi={0};
	nfi.cbSize=sizeof(nfi);
	nfi.hWnd=hWnd;
	nfi.uID=TASKTRAY_ICON_UID;
	lstrcpy(nfi.szTip,"y");

	Shell_NotifyIcon(NIM_DELETE,&nfi);
}

void showBaloon(HWND hWnd,const char* title,const char* content) {
	NOTIFYICONDATA nfi={0};
	nfi.cbSize=sizeof(nfi);
	nfi.hWnd=hWnd;
	nfi.uID=TASKTRAY_ICON_UID;
	nfi.uFlags=NIF_INFO;
	lstrcpy(nfi.szInfo,content);
	nfi.uTimeout=10000;
	lstrcpy(nfi.szInfoTitle,title);
	Shell_NotifyIcon(NIM_MODIFY,&nfi);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message,
                         WPARAM wParam, LPARAM lParam) {
	static HWND hNextWnd;
	static int chainUpdateFlag;
	static UINT taskbarRestartMessage;
	static HMENU hPopupMenu,hMenuTemp;
	static HFONT hFont;
	static config_t config;

	if(message==taskbarRestartMessage) {
		makeTaskTrayIcon(hWnd);
	} else switch (message) {
		case WM_CREATE:
			loadConfig(&config);
			hFont=CreateFont(-14,0,0,0,400,0,0,0,128,3,2,1,32,"lr SVbN");
			make_ui(hWnd,hFont);
			chainUpdateFlag=1;
			hNextWnd=SetClipboardViewer(hWnd);
			SetTimer(hWnd,TIMER_UPDATE_CBC,30000,NULL);
			makeTaskTrayIcon(hWnd);
			taskbarRestartMessage=RegisterWindowMessage("TaskbarCreated");
			hMenuTemp=LoadMenu(GetModuleHandle(NULL),"THEMENU");
			hPopupMenu=GetSubMenu(hMenuTemp,0);
			break;
		case WM_TIMER:
			if(wParam==TIMER_UPDATE_CBC) {
				chainUpdateFlag=1;
				ChangeClipboardChain(hWnd,hNextWnd);
				hNextWnd=SetClipboardViewer(hWnd);
			}
			break;
		case WM_DRAWCLIPBOARD:
			if(hNextWnd)SendMessage(hNextWnd,message,wParam,lParam);
			if(!chainUpdateFlag) {
				char prime[32];
				if(primeJudge(hWnd,prime,&config)) {
					showBaloon(hWnd,prime,"IfI");
				}
			} else chainUpdateFlag=0;
			break;
		case WM_CHANGECBCHAIN:
			if((HWND)wParam==hNextWnd)hNextWnd=(HWND)lParam;
			else if(hNextWnd)SendMessage(hNextWnd,message,wParam,lParam);
			break;
		case MSG_TASKTRAY:
			if(lParam==WM_RBUTTONUP) {
				POINT mpos;
				GetCursorPos(&mpos);
				SetForegroundWindow(hWnd);
				TrackPopupMenu(hPopupMenu,TPM_RIGHTALIGN | TPM_BOTTOMALIGN,
					mpos.x,mpos.y,0,hWnd,NULL);
			}
			break;
		case WM_COMMAND:
			switch (LOWORD(wParam)) {
				case MENU_ID_CONFIG:
					applyConfigToUI(hWnd,&config);
					ShowWindow(hWnd,SW_SHOW);
					break;
				case MENU_ID_EXIT:
					SendMessage(hWnd,WM_DESTROY,0,0);
					break;
				case IDOK:
					if(HIWORD(wParam)==BN_CLICKED) {
						getConfigFromUI(hWnd,&config);
						saveConfig(&config);
						ShowWindow(hWnd,SW_HIDE);
					}
					break;
				case IDC_BUTTON_APPLY:
					if(HIWORD(wParam)==BN_CLICKED) {
						getConfigFromUI(hWnd,&config);
						saveConfig(&config);
					}
					break;
				case IDCANCEL:
					if(HIWORD(wParam)==BN_CLICKED) {
						ShowWindow(hWnd,SW_HIDE);
					}
					break;
				case IDC_BUTTON_FILESELECT:
					if(HIWORD(wParam)==BN_CLICKED) {
						OPENFILENAME ofn={0};
						char fileName[1024];
						ofn.lStructSize=sizeof(ofn);
						ofn.hwndOwner=hWnd;
						ofn.lpstrFilter="t@C(*.wav)\0*.wav\0\0";
						ofn.nFilterIndex=1;
						ofn.lpstrFile=fileName;
						ofn.nMaxFile=sizeof(fileName);
						ofn.Flags=OFN_EXPLORER | OFN_FILEMUSTEXIST |
							OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_NOCHANGEDIR |
							OFN_PATHMUSTEXIST;
						ofn.lpstrDefExt="wav";
						if(GetOpenFileName(&ofn)) {
							SetWindowText(GetDlgItem(hWnd,IDC_EDIT_VOICEFILE),fileName);
						}
					}
					break;
				default:
					return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CLOSE:
			ShowWindow(hWnd,SW_HIDE);
			break;
		case WM_DESTROY:
			DeleteObject(hFont);
			deleteTaskTrayIcon(hWnd);
			DestroyMenu(hPopupMenu);
			DestroyMenu(hMenuTemp);
			KillTimer(hWnd,TIMER_UPDATE_CBC);
			ChangeClipboardChain(hWnd,hNextWnd);
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}

	return 0;
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	WNDCLASSEX wcex;
	HWND hWnd;
	HBRUSH h3DFaceBrush;
	int ret;

	h3DFaceBrush=CreateSolidBrush(GetSysColor(COLOR_3DFACE));

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style          = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc    = (WNDPROC)WndProc;
	wcex.cbClsExtra     = 0;
	wcex.cbWndExtra     = 0;
	wcex.hInstance      = hInstance;
	wcex.hIcon          = LoadImage(hInstance,"MAINICON",IMAGE_ICON,32,32,LR_SHARED);
	wcex.hCursor        = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName   = NULL;
	wcex.lpszClassName  = szWindowClass;
	wcex.hbrBackground  = h3DFaceBrush;
	wcex.hIconSm        = hIconSm=LoadImage(hInstance,"MAINICON",IMAGE_ICON,16,16,LR_SHARED);

	RegisterClassEx(&wcex);

	hWnd = CreateWindow(szWindowClass,
		szWindowTitle, WS_DLGFRAME | WS_CAPTION | WS_SYSMENU | WS_VISIBLE,
		CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
		NULL, NULL, hInstance, NULL);

	if (!hWnd)return FALSE;

	ShowWindow(hWnd, SW_HIDE);
	UpdateWindow(hWnd);

	while((ret=GetMessage(&msg, NULL, 0, 0)) && ret!=-1) {
		if(!IsDialogMessage(hWnd,&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	DeleteObject(h3DFaceBrush);

	return msg.wParam;
}
